@@ -352,7 +352,7 @@ typedef struct {
   int pure_b; //!< >0 if B-modes have been purified.
   flouble *mask; //!< Field's mask (an array of \p npix values).
   fcomplex **a_mask; //!< Fourier transform of the mask. Only computed if E or B are purified.
-  int pol; //!< >0 if field is spin-2 (otherwise it's spin-0).
+  int spin; //!< field's spin (>=0).
   int nmaps; //!< Number of maps in the field (2 for spin-2, 1 for spin-0).
   flouble **maps; //!< Observed field values. When initialized, these maps are already multiplied by the mask, contaminant deprojected and purified if requested.
   fcomplex **alms; //!< Fourier-transfoms of the maps.
@@ -361,6 +361,8 @@ typedef struct {
   fcomplex ***a_temp; //!< Fourier-transfomrs of template maps (mask-multiplied AND purified if requested).
   gsl_matrix *matrix_M; //!< Inverse contaminant covariance matrix (see scientific documentation or companion paper).
   nmt_k_function *beam; //!< Function defining a circularly-symmetric beam function. Power spectra will be beam-deconvolved.
+  int lite; //!< lightweight field (no maps, temp, a_temp or a_mask)
+  int mask_only; //!< this field only contains a mask, and beam. No alms, maps or anything else.
 } nmt_field_flat;
 
 /**
@@ -377,7 +379,7 @@ void nmt_field_flat_free(nmt_field_flat *fl);
  * @param lx Length of the x dimension (in steradians).
  * @param ly Length of the y dimension (in steradians).
  * @param mask Field's mask (an array of \p nx * \p ny values).
- * @param pol >0 if this is a spin-2 field (spin-0 otherwise).
+ * @param spin Field's spin.
  * @param maps Observed field values BEFORE multiplying by the mask
           (this is irrelevant for binary masks).
  * @param ntemp Number of contaminant templates affecting this field.
@@ -396,12 +398,18 @@ void nmt_field_flat_free(nmt_field_flat *fl);
 	  eigenvalue will be discarded.
  * @param masked_input if not 0, input maps and templates have already been masked.
           This is not advisable if using purification.
+ * @param is_lite if not 0, only the map alms and the mask will be stored. You can then
+          use this field to compute the standard pseudo-C_ell with deprojection and purification,
+          but you won't be able to compute the deprojection bias or examine any maps.
+ * @param mask_only if not 0, this field will only store a mask and a beam. You will
+          be able to use it to compute the PCL and covariance mode coupling matrices, but that's
+          it (no actual power spectra, deprojection biases etc.).
  */
 nmt_field_flat *nmt_field_flat_alloc(int nx,int ny,flouble lx,flouble ly,
-				     flouble *mask,int pol,flouble **maps,int ntemp,flouble ***temp,
+				     flouble *mask,int spin,flouble **maps,int ntemp,flouble ***temp,
 				     int nl_beam,flouble *l_beam,flouble *beam,
-				     int pure_e,int pure_b,double tol_pinv,int masked_input);
-
+				     int pure_e,int pure_b,double tol_pinv,int masked_input,
+                                     int is_lite,int mask_only);
 /**
  * @brief Gaussian realizations of flat-sky fields
  *
@@ -490,6 +498,7 @@ nmt_curvedsky_info *nmt_curvedsky_info_copy(nmt_curvedsky_info *cs_in);
  * such that the map actually fits on the sphere (i.e. theta0-(ny-1)*Dtheta >=0).
  * @param is_healpix is this HEALPix pixelization.
  * @param nside if is_healpix, this should be the HEALPix Nside parameter.
+ * @param lmax_sht maximum multipole up to which spherical harmonic transforms will be computed.
  * @param nx0 number of pixels in the phi direction.
  * @param ny0 number of pixels in the theta direction.
  * @param Dtheta pixel size in the theta direction. In radians. Must be positive.
@@ -539,7 +548,7 @@ typedef struct {
   int pure_b; //!< >0 if B-modes have been purified
   flouble *mask; //!< Field's mask (an array of \p npix values).
   fcomplex **a_mask; //!< Spherical transform of the mask. Only computed if E or B are purified.
-  int pol; //!< >0 if field is spin-2 (otherwise it's spin-0).
+  int spin; //!< field's spin (>=0).
   int nmaps; //!< Number of maps in the field (2 for spin-2, 1 for spin-0).
   flouble **maps; //!< Observed field values. When initialized, these maps are already multiplied by the mask, contaminant-deprojected and purified if requested.
   fcomplex **alms; //!< Spherical harmonic transfoms of the maps.
@@ -548,6 +557,8 @@ typedef struct {
   fcomplex ***a_temp; //!< Spherical harmonic transfomrs of template maps (mask-multiplied AND purified if requested).
   gsl_matrix *matrix_M; //!< Inverse contaminant covariance matrix (see scientific documentation or companion paper).
   flouble *beam; //!< Field's beam (defined on all multipoles up to \p lmax).
+  int lite; //!< lightweight field (no maps, temp, a_temp or a_mask)
+  int mask_only; //!< this field only contains a mask, and beam. No alms, maps or anything else.
 } nmt_field;
 
 /**
@@ -561,7 +572,7 @@ void nmt_field_free(nmt_field *fl);
  * Builds an nmt_field structure from input maps and resolution parameters.
  * @param cs curved sky geometry info.
  * @param mask Field's mask.
- * @param pol >0 if this is a spin-2 field (spin-0 otherwise).
+ * @param spin Field's spin.
  * @param maps Observed field values BEFORE multiplying by the mask
           (this is irrelevant for binary masks).
  * @param ntemp Number of contaminant templates affecting this field.
@@ -584,11 +595,17 @@ void nmt_field_free(nmt_field *fl);
  * @param niter number of iterations when computing alms (for all transforms other than the mask's).
  * @param masked_input if not 0, input maps and templates have already been masked.
           This is not advisable if using purification.
+ * @param is_lite if not 0, only the map alms and the mask will be stored. You can then
+          use this field to compute the standard pseudo-C_ell with deprojection and purification,
+          but you won't be able to compute the deprojection bias or examine any maps.
+ * @param mask_only if not 0, this field will only store a mask and a beam. You will
+          be able to use it to compute the PCL and covariance mode coupling matrices, but that's
+          it (no actual power spectra, deprojection biases etc.).
  */
-nmt_field *nmt_field_alloc_sph(nmt_curvedsky_info *cs,flouble *mask,int pol,flouble **maps,
+nmt_field *nmt_field_alloc_sph(nmt_curvedsky_info *cs,flouble *mask,int spin,flouble **maps,
 			       int ntemp,flouble ***temp,flouble *beam,
 			       int pure_e,int pure_b,int n_iter_mask_purify,double tol_pinv,
-			       int niter,int masked_input);
+			       int niter,int masked_input,int is_lite,int mask_only);
 
 /**
  * @brief nmt_field constructor from file.
@@ -596,11 +613,11 @@ nmt_field *nmt_field_alloc_sph(nmt_curvedsky_info *cs,flouble *mask,int pol,flou
  * Builds an nmt_field structure from data written in files.
  * @param is_healpix is the map stored in healpix format?
  * @param fname_mask Path to FITS file containing the field's mask (single HEALPix map).
- * @param pol >0 if this is a spin-2 field (spin-0 otherwise).
+ * @param spin Field's spin.
  * @param fname_maps Path to FITS file containing the field's observed maps
-          (1(2) maps if \p pol=0(1)).
+          (1(2) maps if \p spin=0(!=0)).
  * @param fname_temp Path to FITS file containing the field's contaminant templates.
-          If \p pol > 0, spin-2 is assumed, and the file should contain an even number
+          If \p spin > 0, the file should contain an even number
           of files. Each consecutive pair of maps will be interpreted as the Q and U
 	  components of a given contaminant. Pass "none" if you don't want any contaminants.
  * @param fname_beam Path to ASCII file containing the field's beam. The file should
@@ -622,7 +639,7 @@ nmt_field *nmt_field_alloc_sph(nmt_curvedsky_info *cs,flouble *mask,int pol,flou
  * @param niter number of iterations when computing alms (other than the mask's).
  */
 nmt_field *nmt_field_read(int is_healpix,char *fname_mask,char *fname_maps,char *fname_temp,
-			  char *fname_beam,int pol,int pure_e,int pure_b,
+			  char *fname_beam,int spin,int pure_e,int pure_b,
 			  int n_iter_mask_purify,double tol_pinv,int niter);
 
 /**
@@ -971,6 +988,7 @@ void nmt_workspace_update_binning(nmt_workspace *w,
  * @brief Updates the beams associated to this workspace.
  *
  * Also recomputes the binned MCM and its inverse
+ * @param w workspace.
  * @param nl1 Number of elements of b1.
  * @param b1 First field's beam (harmonic space). One element per multipole.
  * @param nl2 Number of elements of b1.
@@ -1154,7 +1172,10 @@ nmt_covar_workspace_flat *nmt_covar_workspace_flat_init(nmt_field_flat *fla1,nmt
  * and two nmt_covar_workspace_flat structures.
  * @param cw nmt_covar_workspace_flat structure containing the information necessary to compute the
           covariance matrix.
- * @param pol_X whether field X is spin-2
+ * @param spin_a field a spin.
+ * @param spin_b field b spin.
+ * @param spin_c field c spin.
+ * @param spin_d field d spin.
  * @param wa nmt_workspace_flat structure containing the mode-coupling matrix for the first power spectra (between fields a and b).
  * @param wb nmt_workspace_flat structure containing the mode-coupling matrix for the second power spectra (between fields c and d).
  * @param nl Number of multipoles in which input power spectra are computed.
@@ -1167,7 +1188,7 @@ nmt_covar_workspace_flat *nmt_covar_workspace_flat_init(nmt_field_flat *fla1,nmt
           where nbpw_X and ncls_X are the number of bandpowers and different power spectra in the X-th set of fields.
  */
 void nmt_compute_gaussian_covariance_flat(nmt_covar_workspace_flat *cw,
-					  int pol_a,int pol_b,int pol_c,int pol_d,
+					  int spin_a,int spin_b,int spin_c,int spin_d,
 					  nmt_workspace_flat *wa,nmt_workspace_flat *wb,
 					  int nl,flouble *larr,
 					  flouble **clac,flouble **clad,
@@ -1219,7 +1240,10 @@ nmt_covar_workspace *nmt_covar_workspace_init(nmt_field *fla1,nmt_field *fla2,
  * and a nmt_covar_workspace structure.
  * @param cw nmt_covar_workspace structure containing the information necessary to compute the
           covariance matrix.
- * @param pol_X whether field X is spin-2
+ * @param spin_a field a spin.
+ * @param spin_b field b spin.
+ * @param spin_c field c spin.
+ * @param spin_d field d spin.
  * @param wa nmt_workspace structure containing the mode-coupling matrix for the first power spectra.
  * @param wb nmt_workspace structure containing the mode-coupling matrix for the second power spectra.
  * @param clac Cross-power spectra between field 1 in the first set and field 1 in the second set (ac)
@@ -1231,12 +1255,40 @@ nmt_covar_workspace *nmt_covar_workspace_init(nmt_field *fla1,nmt_field *fla2,
           where nbpw_X and ncls_X are the number of bandpowers and different power spectra in the X-th set of fields.
  */
 void  nmt_compute_gaussian_covariance(nmt_covar_workspace *cw,
-				      int pol_a,int pol_b,int pol_c,int pol_d,
+				      int spin_a,int spin_b,int spin_c,int spin_d,
 				      nmt_workspace *wa,nmt_workspace *wb,
 				      flouble **clac,flouble **clad,
 				      flouble **clbc,flouble **clbd,
 				      flouble *covar_out);
 
+/**
+ * @brief Compute full-sky Gaussian covariance matrix
+ *
+ * Computes the covariance matrix for two sets of power spectra given input predicted spectra
+ * and a nmt_covar_workspace structure. Calculation done for the mode-coupled pseudo-Cls.
+ * @param cw nmt_covar_workspace structure containing the information necessary to compute the
+          covariance matrix.
+ * @param spin_a field a spin.
+ * @param spin_b field b spin.
+ * @param spin_c field c spin.
+ * @param spin_d field d spin.
+ * @param wa nmt_workspace structure containing the mode-coupling matrix for the first power spectra.
+ * @param wb nmt_workspace structure containing the mode-coupling matrix for the second power spectra.
+ * @param clac Cross-power spectra between field 1 in the first set and field 1 in the second set (ac)
+          All power spectra should be defined for all ell < lmax.
+ * @param clad Cross-power spectra between field 1 in the first set and field 2 in the second set (ad)
+ * @param clbc Cross-power spectra between field 2 in the first set and field 1 in the second set (bc)
+ * @param clbd Cross-power spectra between field 2 in the first set and field 2 in the second set (bd)
+ * @param covar_out flattened covariance matrix. Should be allocated to shape [ncls_1 * nbpw_1 * ncls_2 * nbpw_2],
+          where nbpw_X and ncls_X are the number of bandpowers and different power spectra in the X-th set of fields.
+ */
+void  nmt_compute_gaussian_covariance_coupled(nmt_covar_workspace *cw,
+                                              int spin_a,int spin_b,int spin_c,int spin_d,
+                                              nmt_workspace *wa,nmt_workspace *wb,
+                                              flouble **clac,flouble **clad,
+                                              flouble **clbc,flouble **clbd,
+                                              flouble *covar_out);
+
 /**
  * @brief Saves nmt_workspace structure to file
  *
