@@ -155,6 +155,9 @@ class TestWorkspaceHPX(unittest.TestCase):
         self.mps = np.array(hp.read_map("test/benchmarks/mps.fits",
                                         verbose=False,
                                         field=[0, 1, 2]))
+        self.mps_s1 = np.array(hp.read_map("test/benchmarks/mps_sp1.fits",
+                                           verbose=False,
+                                           field=[0, 1, 2]))
         self.tmp = np.array(hp.read_map("test/benchmarks/tmp.fits",
                                         verbose=False,
                                         field=[0, 1, 2]))
@@ -188,6 +191,71 @@ class TestWorkspaceHPX(unittest.TestCase):
         self.nlbb = nlbb[:3*self.nside]
         self.nlte = nlte[:3*self.nside]
 
+    def test_lite_pure(self):
+        f0 = nmt.NmtField(self.msk, [self.mps[0]])
+        f2l = nmt.NmtField(self.msk, [self.mps[1], self.mps[2]],
+                           purify_b=True, lite=True)
+        f2e = nmt.NmtField(self.msk, None, purify_b=True,
+                           lite=True, spin=2)
+        nlth = np.array([self.nlte, 0*self.nlte])
+        w = nmt.NmtWorkspace()
+        w.compute_coupling_matrix(f0, f2e, self.b)
+        clb = nlth
+        cl = w.decouple_cell(nmt.compute_coupled_cell(f0, f2l),
+                             cl_bias=clb)
+        tl = np.loadtxt("test/benchmarks/bm_nc_yp_c02.txt",
+                        unpack=True)[1:, :]
+        self.assertTrue((np.fabs(cl-tl) <=
+                         np.fmin(np.fabs(cl),
+                                 np.fabs(tl))*1E-5).all())
+
+    def test_lite_cont(self):
+        f0 = nmt.NmtField(self.msk, [self.mps[0]], templates=[[self.tmp[0]]])
+        f2 = nmt.NmtField(self.msk, [self.mps[1], self.mps[2]],
+                          templates=[[self.tmp[1], self.tmp[2]]])
+        f2l = nmt.NmtField(self.msk, [self.mps[1], self.mps[2]],
+                           templates=[[self.tmp[1], self.tmp[2]]])
+        f2e = nmt.NmtField(self.msk, None, lite=True, spin=2)
+        clth = np.array([self.clte, 0*self.clte])
+        nlth = np.array([self.nlte, 0*self.nlte])
+        w = nmt.NmtWorkspace()
+        w.compute_coupling_matrix(f0, f2e, self.b)
+        clb = nlth
+        dlb = nmt.deprojection_bias(f0, f2, clth+nlth)
+        clb += dlb
+        cl = w.decouple_cell(nmt.compute_coupled_cell(f0, f2l),
+                             cl_bias=clb)
+        tl = np.loadtxt("test/benchmarks/bm_yc_np_c02.txt",
+                        unpack=True)[1:, :]
+        tlb = np.loadtxt("test/benchmarks/bm_yc_np_cb02.txt",
+                         unpack=True)[1:, :]
+        self.assertTrue((np.fabs(dlb-tlb) <=
+                         np.fmin(np.fabs(dlb),
+                                 np.fabs(tlb))*1E-5).all())
+        self.assertTrue((np.fabs(cl-tl) <=
+                         np.fmin(np.fabs(cl),
+                                 np.fabs(tl))*1E-5).all())
+
+    def test_spin1(self):
+        prefix = "test/benchmarks/bm_sp1"
+        f0 = nmt.NmtField(self.msk, [self.mps_s1[0]])
+        f1 = nmt.NmtField(self.msk,
+                          [self.mps_s1[1], self.mps_s1[2]],
+                          spin=1)
+        f = [f0, f1]
+
+        for ip1 in range(2):
+            for ip2 in range(ip1, 2):
+                w = nmt.NmtWorkspace()
+                w.compute_coupling_matrix(f[ip1], f[ip2], self.b)
+                cl = w.decouple_cell(nmt.compute_coupled_cell(f[ip1],
+                                                              f[ip2]))[0]
+                tl = np.loadtxt(prefix+'_c%d%d.txt' % (ip1, ip2),
+                                unpack=True)[1]
+                self.assertTrue((np.fabs(cl-tl) <=
+                                 np.fmin(np.fabs(cl),
+                                         np.fabs(tl))*1E-5).all())
+
     def mastest(self, wtemp, wpure, do_teb=False):
         prefix = "test/benchmarks/bm"
         if wtemp:
@@ -373,6 +441,23 @@ class TestWorkspaceHPX(unittest.TestCase):
         compare_bpw_convolution(np.array([clte, cltb]),  w02)
         compare_bpw_convolution(np.array([clee, cleb, cleb, clbb]),  w22)
 
+    def test_lite_errors(self):
+        f0 = nmt.NmtField(self.msk, [self.mps[0]], n_iter=0)
+        fl = nmt.NmtField(self.msk, [self.mps[0]],
+                          templates=[[self.tmp[0]]], n_iter=0,
+                          lite=True)
+        with self.assertRaises(ValueError):  # Needs spin
+            fe = nmt.NmtField(self.msk, None)
+        fe = nmt.NmtField(self.msk, None, spin=0)
+
+        for f in [fl, fe]:
+            with self.assertRaises(RuntimeError):  # No deprojection bias
+                nmt.deprojection_bias(f0, f, np.zeros([1, 3*self.nside]))
+            with self.assertRaises(RuntimeError):  # No deprojection bias
+                nmt.uncorr_noise_deprojection_bias(fl, self.mps[0])
+        with self.assertRaises(RuntimeError):  # No C_l without maps
+            nmt.compute_coupled_cell(f0, fe)
+
     def test_workspace_methods(self):
         w = nmt.NmtWorkspace()
         w.compute_coupling_matrix(self.f0, self.f0, self.b)  # OK init
@@ -492,6 +577,8 @@ class TestWorkspaceFsk(unittest.TestCase):
         self.ly = np.radians(np.fabs(self.ny*self.wcs.wcs.cdelt[1]))
         self.mps = np.array([read_flat_map("test/benchmarks/mps_flat.fits",
                                            i_map=i)[1] for i in range(3)])
+        self.mps_s1 = np.array([read_flat_map("test/benchmarks/mps_sp1_flat.fits",
+                                              i_map=i)[1] for i in range(3)])
         self.tmp = np.array([read_flat_map("test/benchmarks/tmp_flat.fits",
                                            i_map=i)[1] for i in range(3)])
         self.d_ell = 20
@@ -528,6 +615,88 @@ class TestWorkspaceFsk(unittest.TestCase):
         self.nb_good = np.zeros([1, self.b.bin.n_bands])
         self.nb_bad = np.zeros([2, self.b.bin.n_bands])
 
+    def test_lite_pure(self):
+        f0 = nmt.NmtFieldFlat(self.lx, self.ly, self.msk,
+                              [self.mps[0]])
+        f2l = nmt.NmtFieldFlat(self.lx, self.ly, self.msk,
+                               [self.mps[1], self.mps[2]],
+                               purify_b=True, lite=True)
+        f2e = nmt.NmtFieldFlat(self.lx, self.ly, self.msk,
+                               None, purify_b=True,
+                               lite=True, spin=2)
+        nlth = np.array([self.nlte, 0*self.nlte])
+        w = nmt.NmtWorkspaceFlat()
+        w.compute_coupling_matrix(f0, f2e, self.b)
+        clb = w.couple_cell(self.ll, nlth)
+        cl = w.decouple_cell(nmt.compute_coupled_cell_flat(f0,
+                                                           f2l,
+                                                           self.b),
+                             cl_bias=clb)
+        tl = np.loadtxt("test/benchmarks/bm_f_nc_yp_c02.txt",
+                        unpack=True)[1:, :]
+        self.assertTrue((np.fabs(cl-tl) <=
+                         np.fmin(np.fabs(cl),
+                                 np.fabs(tl))*1E-5).all())
+
+    def test_lite_cont(self):
+        f0 = nmt.NmtFieldFlat(self.lx, self.ly, self.msk,
+                              [self.mps[0]],
+                              templates=[[self.tmp[0]]])
+        tmps = [[self.tmp[1], self.tmp[2]]]
+        f2 = nmt.NmtFieldFlat(self.lx, self.ly, self.msk,
+                              [self.mps[1], self.mps[2]],
+                              templates=tmps)
+        f2l = nmt.NmtFieldFlat(self.lx, self.ly, self.msk,
+                               [self.mps[1], self.mps[2]],
+                               templates=tmps, lite=True)
+        f2e = nmt.NmtFieldFlat(self.lx, self.ly, self.msk,
+                               None, lite=True, spin=2)
+        clth = np.array([self.clte, 0*self.clte])
+        nlth = np.array([self.nlte, 0*self.nlte])
+        w = nmt.NmtWorkspaceFlat()
+        w.compute_coupling_matrix(f0, f2e, self.b)
+        clb = w.couple_cell(self.ll, nlth)
+        dlb = nmt.deprojection_bias_flat(f0, f2,
+                                         self.b, self.ll,
+                                         clth+nlth)
+        clb += dlb
+        cl = w.decouple_cell(nmt.compute_coupled_cell_flat(f0,
+                                                           f2l,
+                                                           self.b),
+                             cl_bias=clb)
+        tl = np.loadtxt("test/benchmarks/bm_f_yc_np_c02.txt",
+                        unpack=True)[1:, :]
+        tlb = np.loadtxt("test/benchmarks/bm_f_yc_np_cb02.txt",
+                         unpack=True)[1:, :]
+        self.assertTrue((np.fabs(dlb-tlb) <=
+                         np.fmin(np.fabs(dlb),
+                                 np.fabs(tlb))*1E-5).all())
+        self.assertTrue((np.fabs(cl-tl) <=
+                         np.fmin(np.fabs(cl),
+                                 np.fabs(tl))*1E-5).all())
+
+    def test_spin1(self):
+        prefix = "test/benchmarks/bm_f_sp1"
+        f0 = nmt.NmtFieldFlat(self.lx, self.ly, self.msk,
+                              [self.mps_s1[0]])
+        f1 = nmt.NmtFieldFlat(self.lx, self.ly, self.msk,
+                              [self.mps_s1[1], self.mps_s1[2]],
+                              spin=1)
+        f = [f0, f1]
+
+        for ip1 in range(2):
+            for ip2 in range(ip1, 2):
+                w = nmt.NmtWorkspaceFlat()
+                w.compute_coupling_matrix(f[ip1], f[ip2], self.b)
+                cl = w.decouple_cell(nmt.compute_coupled_cell_flat(f[ip1],
+                                                                   f[ip2],
+                                                                   self.b))[0]
+                tl = np.loadtxt(prefix+'_c%d%d.txt' % (ip1, ip2),
+                                unpack=True)[1]
+                self.assertTrue((np.fabs(cl-tl) <=
+                                 np.fmin(np.fabs(cl),
+                                         np.fabs(tl))*1E-5).all())
+
     def mastest(self, wtemp, wpure, do_teb=False):
         prefix = "test/benchmarks/bm_f"
         if wtemp:
